﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace BackgroundWorkerExample
{
    public partial class Form1 : Form
    {
        private BackgroundWorker bw;
        public Form1()
        {
            InitializeComponent();
            bw = new BackgroundWorker();
            bw.DoWork += Bw_DoWork;
            bw.RunWorkerCompleted += Bw_RunWorkerCompleted;
            bw.ProgressChanged += Bw_ProgressChanged;
            //Make BackgroundWorker to report progress
            bw.WorkerReportsProgress = true;
            //Allow cancelation
            bw.WorkerSupportsCancellation = true;
        }

        private void Bw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressBar.Value = e.ProgressPercentage;
        }

        private void Bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled)
                MessageBox.Show("Canceled");
            else if (e.Error != null)
                MessageBox.Show("Error: " + e.Error.ToString());
            else
                MessageBox.Show("Work completed\n" + e.Result);
        }

        private void Bw_DoWork(object sender, DoWorkEventArgs e)
        {
            ThreadsArguments arguments = (ThreadsArguments)e.Argument;
            for (int i = 0; i < arguments.Iterations; i++)
            {
                Console.WriteLine("I'm doing operation no: " + i);
                System.Threading.Thread.Sleep(arguments.SleepingTime);
                bw.ReportProgress((i + 1) * 100 / arguments.Iterations);
                if (bw.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }
            }
            string message = "In total " + (arguments.Iterations * arguments.SleepingTime) + " milisekund";
            e.Result = message;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
        }

        private void StartButton_Click(object sender, EventArgs e)
        {
            try
            {
                //Get the data from form
                int iter = (int)IterationsUD.Value;
                int sleep = (int)SleepUD.Value;
                //and make arguments object to pass to the worker
                ThreadsArguments param = new ThreadsArguments() { Iterations = iter, SleepingTime = sleep };
               
                bw.RunWorkerAsync(param);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            bw.CancelAsync();
        }
    }
}
