﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace WPFThreads
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        #region unsafe counter
        private void UnsafeCounter()
        {
            for (int number = 0; number < 21; number++)
            {
                counterLabel.Content = number.ToString();

                Thread.Sleep(200);
            }
        }

        #endregion

        #region Thread safe counter
        delegate void WriteTextDel(string tekst);
        private void WriteText(string tekst)
        {
            counterLabel.Content = tekst;
        }
        private void ThreadSafeCounter()
        {
            WriteTextDel writeDel = new WriteTextDel(WriteText);
            for (int number = 0; number < 21; number++)
            {
                if (!counterLabel.Dispatcher.CheckAccess())
                {
                    counterLabel.Dispatcher.Invoke(writeDel, new object[] { number.ToString() });
                }
                else
                {
                    counterLabel.Content = number.ToString();
                }
                Thread.Sleep(200);
            }
        }
        #endregion

        #region Thread safe counter 2
        /// <summary>
        /// Example of safe operation using static method
        /// </summary>
        private void ThreadSafeCounter2()
        {
            for (int number = 0; number < 21; number++)
            {
                ThreadSafeCallsWPF.setLabelContent(counterLabel, number.ToString());
                Thread.Sleep(200);
            }
        }
        #endregion

        #region Unsafe counter 2
        /// <summary>
        /// Example unsafe operation on canvas
        /// </summary>
        private void UnsafeCounterCanvas()
        {
            for (int number = 0; number < 21; number++)
            {
                ThreadSafeCallsWPF.setLabelContent(counterLabel, number.ToString());
                addFigureToCanvasUnsafe();
                Thread.Sleep(200);
            }
        }
        private void addFigureToCanvasUnsafe()
        {
            Random random = new Random();
            int w, h, x, y;

            switch (random.Next(0, 3))
            {
                case 0:
                    {
                        //   Ellipse 
                        w = random.Next(5, 30);
                        h = random.Next(5, 30);
                        x = random.Next(0, (int)(canvas1.ActualWidth - w));
                        y = random.Next(0, (int)(canvas1.ActualHeight - h));
                        Ellipse ellipse = new Ellipse();
                        ellipse.Width = w;
                        ellipse.Height = h;
                        ellipse.Fill = Brushes.Red;
                        canvas1.Children.Add(ellipse);
                        Canvas.SetLeft(ellipse, x);
                        Canvas.SetTop(ellipse, y);

                        break;
                    }
                case 1:
                    {
                        //triangle
                        int x1, x2, x3, y1, y2, y3;
                        x = random.Next(0, (int)(canvas1.ActualWidth - 35));
                        y = random.Next(0, (int)(canvas1.ActualHeight - 35));
                        x1 = random.Next(0, 35);
                        y1 = random.Next(0, 35);
                        x2 = random.Next(0, 35);
                        y2 = random.Next(0, 35);
                        x3 = random.Next(0, 35);
                        y3 = random.Next(0, 35);
                        Polygon polygon = new Polygon();
                        polygon.Fill = Brushes.Blue;
                        polygon.Points.Add(new Point(x1, y1));
                        polygon.Points.Add(new Point(x2, y2));
                        polygon.Points.Add(new Point(x3, y3));
                        canvas1.Children.Add(polygon);
                        Canvas.SetLeft(polygon, x);
                        Canvas.SetTop(polygon, y);
                        break;
                    }
                case 2:
                    {
                        //rectangle
                        w = random.Next(5, 30);
                        h = random.Next(5, 30);
                        x = random.Next(0, (int)(canvas1.ActualWidth - w));
                        y = random.Next(0, (int)(canvas1.ActualHeight - h));
                        Rectangle rectangle = new Rectangle();
                        rectangle.Width = w;
                        rectangle.Height = h;
                        rectangle.Fill = Brushes.Green;
                        canvas1.Children.Add(rectangle);
                        Canvas.SetLeft(rectangle, x);
                        Canvas.SetTop(rectangle, y);
                        break;
                    }
                default:
                    break;
            }
        }

        #endregion

        #region Thread safe counter canvas
        /// <summary>
        /// Thread safe version with method drawing on canvas
        /// </summary>
        private void ThreadSafeCounterCanvas()
        {
            for (int number = 0; number < 21; number++)
            {
                ThreadSafeCallsWPF.setLabelContent(counterLabel, number.ToString());
                addFigureToCanvas(); //safe version
                Thread.Sleep(200);
            }
        }
        #endregion

        /// <summary>
        /// helping method choosing random color
        /// </summary>
        /// <returns></returns>
        private Color RandomColor()
        {
            Random random = new Random();
            byte r = (byte)random.Next(0, 255);
            byte g = (byte)random.Next(0, 255);
            byte b = (byte)random.Next(0, 255);

            return new Color() { R = r, G = g, B = b, A = 255 };
        }

        /// <summary>
        /// Rysowanie po canvasie w sposób thread safe
        /// </summary>
        private void addFigureToCanvas()
        {
            Random random = new Random();
            int w, h, x, y;
            SolidColorBrush scb = new SolidColorBrush(RandomColor());

            //uncomment below
            //scb.Freeze(); //this is freezable object, so we must freez it to not change in other thread
            switch (random.Next(0, 3))
            {
                case 0:
                    {
                        //   Ellipse 
                        w = random.Next(5, 30);
                        h = random.Next(5, 30);
                        x = random.Next(0, (int)(canvas1.ActualWidth - w));
                        y = random.Next(0, (int)(canvas1.ActualHeight - h));
                        ThreadSafeCallsWPF.drawEllipse(canvas1, w, h, x, y, scb);
                        break;
                    }
                case 1:
                    {
                        //triangle
                        int x1, x2, x3, y1, y2, y3;
                        x = random.Next(0, (int)(canvas1.ActualWidth - 35));
                        y = random.Next(0, (int)(canvas1.ActualHeight - 35));
                        x1 = random.Next(0, 35);
                        y1 = random.Next(0, 35);
                        x2 = random.Next(0, 35);
                        y2 = random.Next(0, 35);
                        x3 = random.Next(0, 35);
                        y3 = random.Next(0, 35);
                        ThreadSafeCallsWPF.drawTriangle(canvas1, x, y, x1, y1, x2, y2, x3, y3, scb);
                        break;
                    }
                case 2:
                    {
                        //rectangle
                        w = random.Next(5, 30);
                        h = random.Next(5, 30);
                        x = random.Next(0, (int)(canvas1.ActualWidth - w));
                        y = random.Next(0, (int)(canvas1.ActualHeight - h));
                        ThreadSafeCallsWPF.drawRectangle(canvas1, w, h, x, y, scb);
                        break;
                    }
                default:
                    break;
            }
        }

        #region thread safe counter 3
        /// <summary>
        /// Thread version z using the parent thread's synchronization context
        /// </summary>
        /// <param name="par">Through the parameter, we pass the synchronization context of the parent thread</param>
        private void ThreadSafeCounter3(object par)
        {
            DispatcherSynchronizationContext context = par as DispatcherSynchronizationContext;
            for (int number = 0; number < 21; number++)
            {
                context.Send((object s) => { counterLabel.Content = s as string; }, number.ToString());
              //  context.Post((object s) => { counterLabel.Content = s as string; }, number.ToString());
                Thread.Sleep(200);
            }
        }
        #endregion

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            //version zero without threads
            //UnsafeCounter();

            //version first
            //Thread thcount = new Thread(UnsafeCounter);
            //thcount.Start();

            //second version
            //Thread thcount = new Thread(ThreadSafeCounter);
            //thcount.Start();

            //third version
            //Thread thcount = new Thread(ThreadSafeCounter2);
            //thcount.Start();

            //fourth version - drowing on canvas
            //Thread thcount = new Thread(UnsafeCounterCanvas);
            //thcount.Start();

            //fifth version with STA but still unsafe
            //Thread thcount = new Thread(UnsafeCounterCanvas);
            //thcount.SetApartmentState(ApartmentState.STA); 
            //thcount.Start();

            //sixth version with safe method drawing on canvas
            //Thread thcount = new Thread(ThreadSafeCounterCanvas);
            ////thcount.SetApartmentState(ApartmentState.STA); //when we have synchronization STA is not necessary
            //thcount.Start();

            //seventh version with synchronization context
          //  Thread thcount = new Thread(ThreadSafeCounter3);
         //   thcount.Start(SynchronizationContext.Current);
          //  thcount.Join(); //what happens when we wait for the thread to finish? What if we use Post instead of Send?
        }
    }
}
