﻿using System;
using System.Collections.Generic;
//using System.Drawing;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace WPFThreads
{
    class ThreadSafeCallsWPF
    {
        public static void setLabelContent(Label label, object content)
        {
            if (!label.Dispatcher.CheckAccess())
            {
                label.Dispatcher.Invoke(new Action(() => { label.Content = content; }), System.Windows.Threading.DispatcherPriority.ApplicationIdle);
            }
            else
            {
                label.Content = content;
            }
        }

        private delegate void drawEllipseDelegate(Canvas canvas, double w, double h, double x, double y, Brush brush);
        public static void drawEllipse(Canvas canvas, double w, double h, double x, double y, Brush brush)
        {

            if (!canvas.Dispatcher.CheckAccess())
            {
                canvas.Dispatcher.Invoke(new drawEllipseDelegate(drawEllipse), new object[] { canvas, w, h, x, y, brush });
            }
            else
            {
                Ellipse ellipse = new Ellipse();
                ellipse.Width = w;
                ellipse.Height = h;
                ellipse.Fill = brush;
                canvas.Children.Add(ellipse);
                Canvas.SetLeft(ellipse, x);
                Canvas.SetTop(ellipse, y);
            }
        }

        private delegate void drawTriangleDelegate(Canvas canvas, double x, double y, double x1, double y1, double x2, double y2, double x3, double y3, System.Windows.Media.Brush brush);
        public static void drawTriangle(Canvas canvas, double x, double y, double x1, double y1, double x2, double y2, double x3, double y3, System.Windows.Media.Brush brush)
        {

            if (!canvas.Dispatcher.CheckAccess())
            {
                canvas.Dispatcher.Invoke(new drawTriangleDelegate(drawTriangle), new object[] { canvas, x, y, x1, y1, x2, y2, x3, y3, brush });
            }
            else
            {
                Polygon polygon = new Polygon();
                polygon.Fill = brush;
                polygon.Points.Add(new Point(x1, y1));
                polygon.Points.Add(new Point(x2, y2));
                polygon.Points.Add(new Point(x3, y3));
                canvas.Children.Add(polygon);
                Canvas.SetLeft(polygon, x);
                Canvas.SetTop(polygon, y);
            }

        }

        private delegate void drawRectangleDelegate(Canvas canvas, double w, double h, double x, double y, Brush brush);
        public static void drawRectangle(Canvas canvas, double w, double h, double x, double y, Brush brush)
        {

            if (!canvas.Dispatcher.CheckAccess())
            {
                canvas.Dispatcher.Invoke(new drawRectangleDelegate(drawRectangle), new object[] { canvas, w, h, x, y, brush });
            }
            else
            {
                Rectangle rectangle = new Rectangle();
                rectangle.Width = w;
                rectangle.Height = h;
                rectangle.Fill = brush;
                canvas.Children.Add(rectangle);
                Canvas.SetLeft(rectangle, x);
                Canvas.SetTop(rectangle, y);
            }

        }

    }
}
